from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.utils.translation import ugettext_lazy as _

class CountManager(models.Manager):
    def get_for_object(self, obj):
        ctype = ContentType.objects.get_for_model(obj)
        return self.get_or_create(content_type=ctype, object_id=obj.id)[0]

    def get_for_model(self, model):
        ctype = ContentType.objects.get_for_model(model)
        return self.filter(content_type=ctype)

    def increment(self, ctype_id, object_id):
        ctype = ContentType.objects.get(id=ctype_id)
        counter, created = self.get_or_create(content_type=ctype, object_id=object_id)
        counter.count += 1
        counter.save()
        return counter

    def inrement_for_object(self, obj):
        counter = self.get_for_object(obj)
        counter.counter += 1
        counter.save()
        return counter

class ViewCounter(models.Model):
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField(_('Object ID'))
    object = generic.GenericForeignKey('content_type', 'object_id')
    count = models.PositiveIntegerField(_('Counter'), default = 0)

    objects = CountManager()

    def get_object_title(self):
        return unicode(self.object)
    get_object_title.short_description = _('Object title')


    def get_content_type(self):
        return self.content_type
    get_content_type.short_description = _('Content type')

    class Meta:
        verbose_name = _('View Counter')
        verbose_name_plural = _('View Counters')
        unique_together = (('content_type', 'object_id'),)
        ordering = ('-count',)

    def __unicode__(self):
        return _('Counter for %(object)s = %(count)d') % (self.object, self.count)

class RedirCounter( models.Model ):
    title = models.CharField( _('Title'), max_length = 40, blank=True)
    url = models.CharField(_('Redirect URL'), max_length=255, unique = True)
    count = models.PositiveIntegerField( _('Counter'), default = 0 )

    def __unicode__(self):
        return self.title

    class Meta:
        verbose_name = _('Download Counter')
        verbose_name_plural = _('Download Counters')

class Referer( models.Model ):
    counter = models.ForeignKey(RedirCounter, related_name = 'referers')
    url = models.CharField(_('URL'), max_length = 255)
    count = models.PositiveIntegerField( _('Counter'), default = 0 )
    update_date = models.DateTimeField( editable = False, auto_now = True )

    def __unicode__(self):
        return u'To %s from %s - %s' % (self.counter, self.url, self.count)

    class Meta:
        verbose_name = _('Referer')
        verbose_name_plural = _('Referers')
        ordering = ('-update_date',)
        get_latest_by = 'update_date'




from django.contrib import admin
from models import *

class RefererInline(admin.TabularInline):
    model = Referer

_list_display = ('id', 'title', 'url', 'count')
admin.site.register(RedirCounter,
    inlines = [
        RefererInline,
    ],
    list_display = _list_display,
    list_display_links = _list_display,
)

admin.site.register(Referer,
    list_display = ('id', 'counter', 'url', 'count', 'update_date'),
    date_hierarchy = 'update_date',
    list_filter = ('update_date',),
)

admin.site.register(ViewCounter,
    list_display = ('get_object_title', 'get_content_type', 'count'),
    list_filter = ('content_type',),
)




from django.conf.urls.defaults import *

urlpatterns = patterns('django_counter.views',
     (r'^c/(?P<ctype_id>\d+)/(?P<object_id>\d+)/$', 'count', {}, 'django-counter-count'),
     (r'^r/(?P<counter_id>\d+)/$', 'redir', {}, 'django-counter-redir'),
)




DATABASE_ENGINE = 'sqlite3'
DATABASE_NAME = '/tmp/django.db'
INSTALLED_APPS = [
    'django.contrib.contenttypes',
    'django_counter',
]
ROOT_URLCONF = 'django_counter.urls'
TEMPLATE_DIRS = (
    'src/django_counter/test_templates',
)



from django import template
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django_counter.models import ViewCounter

def counter(object):
    ctype = ContentType.objects.get_for_model(object)
    return {
        'ctype': ctype,
        'object': object,
        'site': Site.objects.get_current()}

register = template.Library()
register.inclusion_tag('counter/counter.html')(counter)

@register.tag('view_count')
def do_count(parser, token):
    """example: {% view_count for app.model object_id as varname %}"""
    try:
        tag, tmp_for, package, obj, tmp_as, var = token.split_contents()
        if tmp_for != 'for': raise ValueError
        if tmp_as != 'as': raise ValueError
        app, model = package.split('.')
    except ValueError:
        raise template.TemplateSyntaxError, "The valid syntax for view_count tag is: 'view_count for app.model object_id as varname'"
    content_type = ContentType.objects.get(app_label__exact=app, model__exact=model)
    return ViewCountNode(content_type=content_type, obj=obj, var=var)


class ViewCountNode(template.Node):
    def __init__(self, content_type, obj, var):
        self.content_type = content_type
        self.obj = obj
        self.var = var

    def render(self, context):
        if self.obj.isdigit():
            object_id = self.obj
        else:
            try:
                object_id = template.resolve_variable(self.obj, context)
            except template.VariableDoesNotExist:
                return ''
        try:
            count = ViewCounter.objects.get(content_type=self.content_type, object_id=object_id).count
        except ViewCounter.DoesNotExist:
            count = 0
        context[self.var] = count
        return ''



from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.test import TestCase, Client
from django.core.urlresolvers import reverse

from pdb import set_trace

from models import *

class Post(models.Model):
    title = models.CharField('Title', max_length = 100)

class CounterTests(TestCase):
    def setUp(self):
        self.ctype = ContentType.objects.get_for_model(Post)
        self.post = Post(title='Blah minor')
        self.post.save()
        super(CounterTests, self).setUp()

    def tearDown(self):
        Post.objects.all().delete()
        super(CounterTests, self).tearDown()

    def testForCounterIncrement(self):
        client = Client()

        counter = ViewCounter.objects.get_for_object(self.post)
        self.assertEqual(0, counter.count)

        response = client.get(
            reverse('django-counter-count', kwargs = dict(
                ctype_id=self.ctype.id, object_id=self.post.id)))
        self.assertEqual(200, response.status_code)

        counter = ViewCounter.objects.get_for_object(self.post)
        self.assertEqual(1, counter.count)

class RedirTests(TestCase):
    def setUp(self):
        super(RedirTests, self).setUp()

    def testRedirByIdFailIfCounterNotExists(self):
        client = Client()

        response = client.get(
            reverse('django-counter-redir', kwargs = dict(
                counter_id=123)))
        self.assertEqual(404, response.status_code)

    def testExistingRedir(self):
        client = Client()

        url = 'http://aartemenko.com'
        counter = RedirCounter(url = url)
        counter.save()

        response = client.get(
            reverse('django-counter-redir', kwargs = dict(
                counter_id=counter.id)),
            follow = False)
        self.assertEqual(302, response.status_code)
        self.assertEqual(url, response['Location'])

        counter = RedirCounter.objects.get(url = url)
        self.assertEqual(1, counter.count)




from django.http import HttpResponseRedirect, HttpResponse
import mimetypes, rfc822, os, stat
from pdb import set_trace
from django.shortcuts import get_object_or_404
from models import RedirCounter, Referer, ViewCounter

from PIL import Image
from StringIO import StringIO

_img  = Image.fromstring('RGB', (1,1), '000')
_ff = StringIO()
_img.save(_ff, format='GIF')
_ff.seek(0)

IMG_MIMETYPE = 'image/gif'
IMG_DATA = _ff.read()

def count(request, ctype_id, object_id):
    ctype_id = int(ctype_id)
    object_id = int(object_id)
    counter = ViewCounter.objects.increment(ctype_id, object_id)

    response = HttpResponse(IMG_DATA, mimetype=IMG_MIMETYPE)
    return response

# We may use
# update counter_downloadcounter as t1 set count = (select sum(t2.count) from counter_referer as t2 where t1.id = t2.counter_id);
# to calculate all total stats, if needed.
#
def redir(request, counter_id):
    referer = request.environ.get('HTTP_REFERER') or 'none'
    counter = get_object_or_404(RedirCounter, id = counter_id)
    ref = counter.referers.filter(url = referer)[:1]
    if not ref:
        ref = Referer(counter = counter, url = referer, count = 0)
    else:
        ref = ref[0]
    ref.count += 1
    ref.save()
    counter.count += 1
    counter.save()
    return HttpResponseRedirect(counter.url)
